import { EncryptionFunction, EncrypterOptions } from '../types';
/**
 * Create a sync encrypt function that can be used to encrypt UTF-8 strings,
 * The return function takes plainText and a key to generate encrypted
 * text based on AES-GCM with IV length of 16 and uses pbkdf2 for
 * generating 32 byte (256 bit) Key both SALT and IV are generated using
 * crypto.randomBytes
 * @default
 * * outputEncoding = 'base64',
 * * stringEncoding = 'utf8',
 * * authTagLength = 16,
 * * ivLength = 12,
 * * saltLength = 32,
 * * iterations = 1,
 * * digest = 'sha256'
 * @example
 * ```
 * const encrypt = makeStringEncrypter({});
 * const cipherText = encrypt('some text', 'some password');
 * ```
 * @note
 * Out put string is concatinates salt,iv,authTags and encryptedtext
 * in this precise order as no delimiters are used decrption function
 * should be created with same salt iv and auth tags length
 */
export declare const makeStringEncrypterSync: (opt: EncrypterOptions) => EncryptionFunction<string>;
