"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.makeStringEncrypter = void 0;
const crypto_1 = require("crypto");
const random_bytes_1 = require("./random-bytes");
const key_length_hint_1 = require("./key-length-hint");
const pbkdf2_1 = require("./pbkdf2");
/**
 * Create an async encrypt function that can be used to encrypt UTF-8 strings,
 * The return function takes plainText and a key to generate encrypted
 * text based on AES-GCM with IV length of 16 and uses pbkdf2 for
 * generating 32 byte (256 bit) Key both SALT and IV are generated using
 * crypto.randomBytes
 * @default
 * * outputEncoding = 'base64',
 * * stringEncoding = 'utf8',
 * * authTagLength = 16,
 * * ivLength = 12,
 * * saltLength = 32,
 * * iterations = 1,
 * * digest = 'sha256'
 * @example
 * ```
 * const encrypt = makeStringEncrypter({});
 * const cipherText = encrypt('some text', 'some password');
 * ```
 * @note
 * Out put string is concatinates salt,iv,authTags and encryptedtext
 * in this precise order as no delimiters are used decrption function
 * should be created with same salt iv and auth tags length
 */
const makeStringEncrypter = ({ algorithm, outputEncoding = 'base64', stringEncoding = 'utf8', authTagLength = 16, ivLength = 12, saltLength = 32, iterations = 1, digest = 'sha256' }) => ((text, password) => __awaiter(void 0, void 0, void 0, function* () {
    const iv = yield random_bytes_1.randomBytes(ivLength);
    const salt = yield random_bytes_1.randomBytes(saltLength);
    const key = yield pbkdf2_1.pbkdf2(password, salt, iterations, key_length_hint_1.keyLengthHint(algorithm), digest);
    const cipher = crypto_1.createCipheriv(algorithm, key, iv, { authTagLength });
    const cipherText = Buffer.concat([cipher.update(text, stringEncoding), cipher.final()]);
    return Buffer.concat([salt, iv, cipher.getAuthTag(), cipherText]).toString(outputEncoding);
}));
exports.makeStringEncrypter = makeStringEncrypter;
